unit WABD_PropEditors;

interface

uses SysUtils, Windows, Dialogs;

   procedure Register;

implementation

uses DsgnIntf, WABD_About, WABD_Objects, WABD_FormEditor, WABD_TableStrEditor,
      WABD_HotSpotEditor;

var
   FormEditor : TWABDFormEditor;
   HotSpotEditor : THSEditForm;

type
   TWABD_Form_Comp_Editor = class(TComponentEditor)
      procedure   ExecuteVerb(Index: Integer); override;
      function    GetVerb(Index: Integer): string; override;
      function    GetVerbCount: Integer; override;
   public
      procedure   DesignForm;
   end;

   TTable_Strings_Editor = class(TPropertyEditor)
      procedure   Edit; override;
      function    GetAttributes: TPropertyAttributes; override;
      function    GetValue: string; override;
   end;

   TWABDEditFormProperty = class(TPropertyEditor)
   public
      procedure Edit; override;
      function  GetAttributes: TPropertyAttributes; override;
      function  GetValue: string; override;
      function  GetName: string; override;
   end;

   TWABDHotSpotProperty = class(TPropertyEditor)
   public
      procedure Edit; override;
      function  GetAttributes: TPropertyAttributes; override;
      function  GetValue: string; override;
   end;


// ************************************************************************

procedure TWABD_Form_Comp_Editor.DesignForm;
begin
   if FormEditor=nil then FormEditor := TWABDFormEditor.Create(nil);
   FormEditor.WABDForm := Component as TWABD_Form;
   FormEditor.Designer := Designer;

   FormEditor.Initialize;
   FormEditor.Show;
end;

procedure TWABD_Form_Comp_Editor.ExecuteVerb(Index: Integer);
begin
   case Index of
      0: DesignForm;
   end;
end;

function TWABD_Form_Comp_Editor.GetVerb(Index: Integer): string;
begin
   case Index of
      0: Result := '&Design WABD Form...';
   end;
end;

function TWABD_Form_Comp_Editor.GetVerbCount: Integer;
begin
   Result := 1;
end;


// ************************************************************************

procedure TTable_Strings_Editor.Edit;
var
   d : TTableStrings_Editor;
begin
   d := TTableStrings_Editor.Create(nil);
   try
      d.EditStrings(TWABD_Table_Strings(GetOrdValue));
      SetOrdValue(GetOrdValue);
   finally
      d.Free;
   end;
end;

function TTable_Strings_Editor.GetAttributes: TPropertyAttributes;
begin
   Result := [paDialog];
end;

function TTable_Strings_Editor.GetValue: string;
begin
   Result := '(Table Strings)';
end;

// ************************************************************************

function TWABDEditFormProperty.GetAttributes: TPropertyAttributes;
begin
   Result := [paDialog];
end;

function TWABDEditFormProperty.GetValue: string;
begin
   Result := '(Click_Me)';
end;

function TWABDEditFormProperty.GetName: string;
begin
   Result := 'Edit Form';
end;

procedure TWABDEditFormProperty.Edit;
begin
   if FormEditor=nil then FormEditor := TWABDFormEditor.Create(nil);
   FormEditor.WABDForm := TWABDEditForm(GetOrdValue).ParForm;
   FormEditor.Designer := Designer;

   FormEditor.Initialize;
   FormEditor.Show;
end;


// ************************************************************************

procedure TWABDHotSpotProperty.Edit;
begin
   if HotSpotEditor=nil then HotSpotEditor := THSEditForm.Create(nil);
   HotSpotEditor.Designer := Designer;
   HotSpotEditor.ParImage := TWABD_HotSpots(GetOrdValue).ParImage;

   HotSpotEditor.Init;
   HotSpotEditor.Show;
end;

function  TWABDHotSpotProperty.GetAttributes: TPropertyAttributes;
begin
   Result := [paDialog];
end;

function  TWABDHotSpotProperty.GetValue: string;
begin
   Result := '(Click_Me)';
end;


// ************************************************************************

procedure Register;
begin
   RegisterComponentEditor(TWABD_Form, TWABD_Form_Comp_Editor);
   RegisterPropertyEditor(TypeInfo(TWABD_Table_Strings), nil, '',
      TTable_Strings_Editor);
   RegisterPropertyEditor(TypeInfo(TWABDAbout), NIL, '', TWABDAboutProperty);
   RegisterPropertyEditor(TypeInfo(TWABDEditForm), NIL, '', TWABDEditFormProperty);
   RegisterPropertyEditor(TypeInfo(TWABD_HotSpots), NIL, '', TWABDHotSpotProperty);
end;



initialization
   FormEditor := nil;
   HotSpotEditor := nil;
finalization
   FormEditor.Free;
   HotSpotEditor.Free;
end.
